// exbrpak.cpp, v1.0 2013/06/10
// coded by asmodean

// contact: 
//   web:   http://asmodean.reverse.net
//   email: asmodean [at] hush.com
//   irc:   asmodean on efnet (irc.efnet.net)

// This tool extracts *.pak archives used by  s.

#include "as-util.h"
#include "as-zlib.h"

struct PAKTRL {
  uint32_t entry_count;
  uint32_t toc_length;
};

struct PAKENTRY {
  uint32_t  offset;
  uint32_t  length;
  uint32_t  original_length;
  uint32_t  filename_length;
  //wchar_t filename[filename_length];
};

int main(int argc, char** argv) {
  if (argc != 2) {
    fprintf(stderr, "exbrpak v1.0, coded by asmodean\n\n");
    fprintf(stderr, "usage: %s <input.pak>\n", argv[0]);
    return -1;
  }

  string in_filename = argv[1];

  int fd = as::open_or_die(in_filename, O_RDONLY | O_BINARY); 

  PAKTRL trl;
  lseek(fd, -(int)sizeof(trl), SEEK_END);
  read(fd, &trl, sizeof(trl));

  uint32_t toc_len  = trl.toc_length;
  auto     toc_buff = new uint8_t[toc_len];
  lseek(fd, -(int)(sizeof(trl) + toc_len), SEEK_END);
  read(fd, toc_buff, toc_len);

  auto p = toc_buff;

  for (uint32_t i = 0; i < trl.entry_count; i++) {
    auto entry = (const PAKENTRY*) p;
    p += sizeof(*entry);

    string filename = as::convert_wchar((wchar_t*)p, entry->filename_length * 2);
    p += entry->filename_length * 2;

    uint32_t len  = entry->length;
    auto     buff = new uint8_t[len];
    _lseeki64(fd, entry->offset, SEEK_SET);
    read(fd, buff, len);

    as::inflate(buff, len, entry->original_length, as::UNCOMPRESS_CHECK);

    as::make_path(filename);
    as::write_file(filename, buff, len);

    delete [] buff;
  }

  delete [] toc_buff;

  close(fd);

  return 0;
}
